"""
aiosched: Yet another Python async scheduler

Copyright (C) 2022 nocturn9x

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   https:www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from typing import Any
from aiosched.errors import SchedulerError
from aiosched.internals.syscalls import (
    suspend,
    schedule,
    current_task,
)


class Event:
    """
    An asynchronous, non thread-safe event
    """

    def __init__(self):
        """
        Object constructor
        """

        self.set = False
        self.waiters = set()

    def reset(self):
        """
        Resets the event's state
        """

        self.__init__()

    async def trigger(self):
        """
        Sets the event, waking up all tasks that called
        wait() on it
        """

        if self.set:
            raise SchedulerError("The event has already been set")
        self.set = True
        for waiter in self.waiters:
            await schedule(waiter)

    async def wait(self) -> Any:
        """
        Waits until the event is set
        """

        self.waiters.add(await current_task())
        await suspend()  # We get unsuspended by trigger()
